import { deepCopy, FormControlProps, FormControlState, IActionParam, MainControl } from '@ibiz-core';

/**
 * @description 表单部件
 * @export
 * @class FormControl
 * @extends {MainControl}
 */
export class FormControl extends MainControl {
  
  /**
   * @description 部件状态
   * @type {FormControlState}
   * @memberof FormControl
   */
  public declare controlState: FormControlState;

  /**
   * @description 表单数据改变
   * @param {string} name 属性名称
   * @param {*} value 属性值
   * @memberof FormControl
   */
  public formDataChange(name: string, value: any){
    this.controlState.data[name] = value;
    console.log(name, value, this.controlState.data);
  }

  /**
   * @description 使用加载功能模块
   * @param {FormControlProps} props 传入的props
   * @return {*} 
   * @memberof FormControl
   */
  public useLoad(props: FormControlProps){
    const { viewSubject, controlName } = this.controlState;
    const load = async (opt: any = {})=>{
      try {
        const loadAction = this.controlState.controlAction.loadAction;
        const { controlService, context, viewParams, showBusyIndicator } = this.controlState;
        const dataRef = toRef(this.controlState, 'data');
        
        if(!loadAction){
          return;
        }
        let _context = deepCopy(context);
        let _viewParams = deepCopy(viewParams);
    
        const response = await controlService.get(loadAction, _context, {viewParams: _viewParams}, showBusyIndicator );
        if (!response.status || response.status !== 200) {
          return
        }
        dataRef.value = response.data;
      } catch (error) {
        // todo 错误异常处理
      }
      
    }

    // 订阅viewSubject,监听load行为
    if(viewSubject){
      let subscription = viewSubject.subscribe(({ tag, action, data }: IActionParam)=>{
        if(Object.is(controlName, tag) && Object.is("load", action) ){
          load(data)
        }
      })
      // 部件卸载时退订viewSubject
      onUnmounted(()=>{
        subscription.unsubscribe();
      })
    }

    return {
      load: load
    }
  }

  /**
   * @description 处理编辑器事件
   * @param {IActionParam} actionParam 行为参数
   * @memberof FormControl
   */
  public handleEditorEvent(actionParam: IActionParam) {
    const { tag, action, data } = actionParam;
    switch (action) {
      case "valueChange":
        this.formDataChange(tag, data);
        break;
      default:
        break;
    }
  }

  /**
   * @description 安装部件所有功能模块的方法
   * @param {FormControlProps} props 传入的Props
   * @param {Function} [emit]
   * @return {*} 
   * @memberof FormControl [emit] 事件
   */
  public moduleInstall(props: FormControlProps, emit?: Function) {
    const superParams = super.moduleInstall(props, emit);
    const { load } = this.useLoad(props)

    const handleEditorEvent = this.handleEditorEvent.bind(this);
    this.handleEditorEvent = (actionParam: IActionParam) => {
      handleEditorEvent(actionParam)
    }

    return {
      ...superParams,
      state: this.controlState,
      load,
      handleEditorEvent: this.handleEditorEvent,
    };
  }
}
